# Time-stamp: <2011-11-17 22:20:57 zaykind> (written by Dmitri Zaykin)

###### USER-DEFINED SECTION
# Set Mean.Increase to zero or a negative value if want to use power instead
Mean.Increase <- -0.3 # effect size (delta-mu) assuming standard deviation of 1
Power <- 0.90 # (at alpha 5%)
Allele.Frequency <- 0.20
Sample.Size <- 860
Num.SNPs <- 2832
Wanted.Prob.To.Capture <- 0.95
# if Num.TruePositives > 1, output probability
# of capturing at least one of them
Num.TruePositives <- 1
Extremal.Index <- 1
###### END OF USER-DEFINED SECTION

cdf.T <- function(a, nc, df) {
  1 - pt(qt(1-a, df, ncp=0), df, ncp=nc)
}

cdfRoot.T <- function(nc, df, a, b) { 
    cdf.T(a,nc,df) - b 
}

# when using two-sided test, be sure to use alpha/2
getNonCentr.T <- function(df, alpha, pow) {
   nc <- uniroot(cdfRoot.T, c(0, 2000), df=df, a=alpha, b=pow)$root
   return (nc)
}

# get allele counts assuming Hardy-Weinberg equilibrium
pA <- Allele.Frequency
pa <- 1-pA
nAA <- Sample.Size * pA^2
naa <- Sample.Size * pa^2
nAa <- Sample.Size * 2*pA*pa

# count for one or more copies of risk allele
nA <- round(nAA + nAa)
# count for the genotype without risk alleles
na <- Sample.Size - nA
# uncomment if want additive model
#nA <- round(2*nAA + nAa)
#na <- 2*Sample.Size - nA

Fy <- function(y, df, nc) {
   1 - pt( qt(1-y, df=df), df=df, ncp=nc)
}

min.Fy <- function(y, df, nc, m) {
   1 - (1 - Fy(y,df,nc))^m
}

eff.rank <- function(i, t, n) {
  (i - 1)*(t*n - 1)/(n - 1) + 1
}

Pr.minY <- function(i,L,th,df,nc,m) {
  ie <- eff.rank(i, th, L)
  min.Fy(qbeta(0.5, ie, th*L-ie+1),  df, nc, m)
}

df <- nA + na - 2
if(Mean.Increase > 0) {
    nc <- Mean.Increase / sqrt(1/nA + 1/na)
} else {
    alpha <- 0.05
    df.harmon <- round(2/(1/nA + 1/na) - 2)
    nc <- getNonCentr.T(df.harmon, alpha/2, Power)
}

cat("Probability to capture an association\n")
for(i in 1:Num.SNPs) {
  rnk.Prob <- Pr.minY(i, Num.SNPs, Extremal.Index, df, nc, Num.TruePositives)
  if(!(i%%20)) cat("... among first", i, "best results =", rnk.Prob, "\n")
  if(rnk.Prob >= Wanted.Prob.To.Capture) break;
}
cat("... among first", i, "best results =", rnk.Prob, "\n")
cat("number of SNPs to follow up on for", 100*Wanted.Prob.To.Capture,
    "% probability =", i, "\n")

if(Mean.Increase <= 0) {
  Effect <- nc * sqrt(1/nA + 1/na)
  cat("Effect size =", Effect, "\n")
}
